#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include <iostream>
#include <string>
#include "LHAPDF/LHAPDF.h"
#include <iterator>
#include <iostream>
#include <cstdlib>

using namespace std;
using namespace LHAPDF;

/*
	Monte Carlo integration.
	We calculate the Sudakov form factor
      and plot it for different upper scales as function of pt

	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
	http://luscher.web.cern.ch/luscher/ranlux/index.html      


*/

extern double t1;
extern double t2;
double alphas(double q )
{
	double QCDlam = 0.2, Qlam0 = 1, pi=3.14159, beta0, Qval, d1, as; 
      int nf=3;
	beta0 =(33-2*nf)/6; 
      if ( q > Qlam0 ) {
           Qval = q; }
         else {
           Qval = Qlam0;
           }
      d1=Qval/QCDlam;
      as= 2.*pi/beta0/log(d1*d1) ;
//      cout << " alphas = " << as << " q = " << q << endl;
      return(as);
}
double Splitting(double z )
{
      double temp;

// here insert the splitting function 

      if ( z > 0.99999 ) {
      cout <<" Splitting large z = " << z<< " P = "<< temp<< endl;
      temp = 0;
	}
      if ( z < 0.000001 ) {
      cout <<" Splitting small z = " << z<< " P = "<< temp<< endl;
      temp = 0;
	}
      return(temp);
}

double suda(double x, double y)
{
	double q2,q, q0=0.1, z1min,z1max,z1,z,temp,d1,d2,d3,result;
	static const double pi = 3.14159265358979323848;
       if ( t1 < q0 ) t1 = q0 ;
       if ( x >= 1 ){ cout << " Suda: argument 1 out of range: "<< x << endl;}
       if ( x <= 0 ){ cout << " Suda: argument 1 out of range: "<< x << endl;}
       if ( y >= 1 ){ cout << " Suda: argument 2 out of range: "<< y << endl;}
       if ( y <= 0 ){ cout << " Suda: argument 2 out of range: "<< y << endl;}
//     	 cout << "Suda:  t2 = "<< t2 << " t1 = "<< t1 << endl;
      
// here calcuate the intrals of the sudakov form factor.
// generate t according to a 1/t distribution using the randaom variable x in the limits t1<t<t2
// generate z1 according to a 1/z1 distribution using the random variable y z1min<z1<z1max
// then z=1-z1 (z1 is used because we have the 1/(1-z) sigularity)
// take sqrt(t)*(1-z) as the argument for alphas


      return(result); 
}

double t1;
double t2;

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 100000;
      int nt, ntmax = 20;
	double x1, y1, sum0, sum00, sigma2, error, ff, sudakov;
	double delta;
      double tmin = 1, tmax = 50; 

#define LVEC 10
	double rvec[LVEC];
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);



//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("t1","sudakov",ntmax,tmin,tmax);
      
      sum0 = sum00 = 0;
      
// 	loop over t1
      delta = (tmax-tmin)/ntmax;

      t2 = tmax ;
      for ( nt = 0; nt < ntmax; nt++ ) {  
      t1 = tmin + delta*nt ;   
//      	cout << " tmax = "<< t2 << " t1 = "<< t1 << " delta " << delta<< endl;
		for ( n1 = 0; n1 < npoints; n1++ ) {
      		ranlxd(rvec,LVEC);
			x1 = rvec[0];
			y1 = rvec[1];
            	ff = suda(x1,y1);
            	sum0 = sum0 + ff;
            	sum00 = sum00 + ff*ff; 
		}
//                                  
      	sum0 = sum0/npoints;
      	sum00 = sum00/npoints;
      	sigma2 = sum00 - sum0*sum0 ;
      	error = sqrt(sigma2/npoints) ;
      	sudakov = exp(-sum0) ;
      	cout << " t2 = "<< t2 << " t1 = "<< t1 << " Delta_S = " << sudakov << " +-" << error << endl;
		histo1->Fill(t1+0.001*delta,sudakov);
	}
//general root settings 
      gROOT->Reset();
      gROOT->SetStyle("Plain");
      gStyle->SetPadTickY(1); // ticks at right side
      gStyle->SetOptStat(0); // get rid of statistics box
      TCanvas *c = new TCanvas("ctest", "" ,0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y
      c -> Divide(1,1);
//enter the first part of the canvas (upper left)
      c -> cd(1);
      gPad->SetLogy();
      histo1 -> Draw();
      c-> Draw();
      c->WaitPrimitive();
      c-> Print("example12.pdf");
      gMyRootApp->SetReturnFromRun(true);
//    write histogramm out to file
      TFile file("output-example12.root","RECREATE");
      histo1->Write();
      file.Close();
	return(0);
}
